/* ----------------------------------------------------------------------    
* Copyright (C) 2010-2014 ARM Limited. All rights reserved.    
*    
* $Date:        12. March 2014
* $Revision: 	V1.4.4
*    
* Project: 	    CMSIS DSP Library    
* Title:		arm_add_q15.c    
*    
* Description:	Q15 vector addition    
*    
* Target Processor: Cortex-M4/Cortex-M3/Cortex-M0
*  
* -------------------------------------------------------------------- */

#include "arm_math.h"

 // * @ingroup groupMath    

 // * @addtogroup BasicAdd    

/**    
 * @brief Q15 vector addition.    
 * @param[in]       *pSrcA points to the first input vector    
 * @param[in]       *pSrcB points to the second input vector    
 * @param[out]      *pDst points to the output vector    
 * @param[in]       blockSize number of samples in each vector    
 * @return none.    
 *    
 * <b>Scaling and Overflow Behavior:</b>    
 * \par    
 * The function uses saturating arithmetic.    
 * Results outside of the allowable Q15 range [0x8000 0x7FFF] will be saturated.    
 */

void arm_add_q15(
  q15_t * pSrcA,
  q15_t * pSrcB,
  q15_t * pDst,
  uint32_t blockSize)
{
  uint32_t blkCnt;                               /* loop counter */

#ifndef ARM_MATH_CM0_FAMILY

// Run the below code for Cortex-M4 and Cortex-M3 
  q31_t inA1, inA2, inB1, inB2;

  // loop Unrolling 
  blkCnt = blockSize >> 2u;

  /* First part of the processing with loop unrolling.  Compute 4 outputs at a time.    
   ** a second loop below computes the remaining 1 to 3 samples. */
  while(blkCnt > 0u)
  {
    // C = A + B 
    // Add and then store the results in the destination buffer. 
    inA1 = *__SIMD32(pSrcA)++;
    inA2 = *__SIMD32(pSrcA)++;
    inB1 = *__SIMD32(pSrcB)++;
    inB2 = *__SIMD32(pSrcB)++;

    *__SIMD32(pDst)++ = __QADD16(inA1, inB1);
    *__SIMD32(pDst)++ = __QADD16(inA2, inB2);

    // Decrement the loop counter 
    blkCnt--;
  }

  /* If the blockSize is not a multiple of 4, compute any remaining output samples here.    
   ** No loop unrolling is used. */
  blkCnt = blockSize % 0x4u;

  while(blkCnt > 0u)
  {
    // C = A + B 
    // Add and then store the results in the destination buffer. 
    *pDst++ = (q15_t) __QADD16(*pSrcA++, *pSrcB++);

    // Decrement the loop counter 
    blkCnt--;
  }

#else

  // Run the below code for Cortex-M0 



  // Initialize blkCnt with number of samples 
  blkCnt = blockSize;

  while(blkCnt > 0u)
  {
    // C = A + B 
    // Add and then store the results in the destination buffer. 
    *pDst++ = (q15_t) __SSAT(((q31_t) * pSrcA++ + *pSrcB++), 16);

    // Decrement the loop counter 
    blkCnt--;
  }

#endif /* #ifndef ARM_MATH_CM0_FAMILY */


}

 // *  end of BasicAdd group    
