/* ----------------------------------------------------------------------    
* Copyright (C) 2010-2014 ARM Limited. All rights reserved.    
*    
* $Date:        12. March 2014
* $Revision: 	V1.4.4
*    
* Project: 	    CMSIS DSP Library    
* Title:		arm_negate_f32.c    
*    
* Description:	Negates floating-point vectors.    
*    
* Target Processor: Cortex-M4/Cortex-M3/Cortex-M0
*  
* ---------------------------------------------------------------------------- */

#include "arm_math.h"

 // * @ingroup groupMath        

/**        
 * @defgroup negate Vector Negate        
 *        
 * Negates the elements of a vector.        
 *        
 * <pre>        
 *     pDst[n] = -pSrc[n],   0 <= n < blockSize.        
 * </pre>        
 *
 * The functions support in-place computation allowing the source and
 * destination pointers to reference the same memory buffer.
 * There are separate functions for floating-point, Q7, Q15, and Q31 data types.
 */

 // * @addtogroup negate        

/**        
 * @brief  Negates the elements of a floating-point vector.        
 * @param[in]  *pSrc points to the input vector        
 * @param[out]  *pDst points to the output vector        
 * @param[in]  blockSize number of samples in the vector        
 * @return none.        
 */

void arm_negate_f32(
  float32_t * pSrc,
  float32_t * pDst,
  uint32_t blockSize)
{
  uint32_t blkCnt;                               /* loop counter */


#ifndef ARM_MATH_CM0_FAMILY

// Run the below code for Cortex-M4 and Cortex-M3 
  float32_t in1, in2, in3, in4;                  /* temporary variables */

  // loop Unrolling 
  blkCnt = blockSize >> 2u;

  /* First part of the processing with loop unrolling.  Compute 4 outputs at a time.        
   ** a second loop below computes the remaining 1 to 3 samples. */
  while(blkCnt > 0u)
  {
    // read inputs from source 
    in1 = *pSrc;
    in2 = *(pSrc + 1);
    in3 = *(pSrc + 2);
    in4 = *(pSrc + 3);

    // negate the input 
    in1 = -in1;
    in2 = -in2;
    in3 = -in3;
    in4 = -in4;

    // store the result to destination 
    *pDst = in1;
    *(pDst + 1) = in2;
    *(pDst + 2) = in3;
    *(pDst + 3) = in4;

    // update pointers to process next samples 
    pSrc += 4u;
    pDst += 4u;

    // Decrement the loop counter 
    blkCnt--;
  }

  /* If the blockSize is not a multiple of 4, compute any remaining output samples here.        
   ** No loop unrolling is used. */
  blkCnt = blockSize % 0x4u;

#else

  // Run the below code for Cortex-M0 

  // Initialize blkCnt with number of samples 
  blkCnt = blockSize;

#endif /* #ifndef ARM_MATH_CM0_FAMILY */

  while(blkCnt > 0u)
  {
    // C = -A 
    // Negate and then store the results in the destination buffer. 
    *pDst++ = -*pSrc++;

    // Decrement the loop counter 
    blkCnt--;
  }
}

 // *  end of negate group        
