/* ----------------------------------------------------------------------    
* Copyright (C) 2010-2014 ARM Limited. All rights reserved.    
*    
* $Date:        12. March 2014
* $Revision: 	V1.4.4
*    
* Project: 	    CMSIS DSP Library    
* Title:		arm_cos_q15.c    
*    
* Description:	Fast cosine calculation for Q15 values.   
*    
* Target Processor: Cortex-M4/Cortex-M3/Cortex-M0
*  
* -------------------------------------------------------------------- */

#include "arm_math.h"
#include "arm_common_tables.h"

 // * @ingroup groupFastMath    

 // * @addtogroup cos    

/**   
 * @brief Fast approximation to the trigonometric cosine function for Q15 data.   
 * @param[in] x Scaled input value in radians.   
 * @return  cos(x).   
 *   
 * The Q15 input value is in the range [0 +0.9999] and is mapped to a radian
 * value in the range [0 2*pi).
 */

q15_t arm_cos_q15(
  q15_t x)
{
  q15_t sinVal;                                  /* Temporary variables for input, output */
  int32_t index;                                 /* Index variables */
  q15_t a, b;                                    /* Four nearest output values */
  q15_t fract;                                   /* Temporary values for fractional values */

  // add 0.25 (pi/2) to read sine table 
  x += 0x2000;
  if(x < 0)
  {   /* convert negative numbers to corresponding positive ones */
      x = x + 0x8000;
  }

  // Calculate the nearest index 
  index = (uint32_t)x >> FAST_MATH_Q15_SHIFT;

  // Calculation of fractional value 
  fract = (x - (index << FAST_MATH_Q15_SHIFT)) << 9;

  // Read two nearest values of input value from the sin table 
  a = sinTable_q15[index];
  b = sinTable_q15[index+1];

  // Linear interpolation process 
  sinVal = (q31_t)(0x8000-fract)*a >> 16;
  sinVal = (q15_t)((((q31_t)sinVal << 16) + ((q31_t)fract*b)) >> 16);

  return sinVal << 1;
}

 // *  end of cos group    
