/* ----------------------------------------------------------------------    
* Copyright (C) 2010-2014 ARM Limited. All rights reserved.    
*    
* $Date:        12. March 2014
* $Revision: 	V1.4.4
*    
* Project: 	    CMSIS DSP Library    
* Title:		arm_sin_q31.c    
*    
* Description:	Fast sine calculation for Q31 values.   
*    
* Target Processor: Cortex-M4/Cortex-M3/Cortex-M0
*  
* -------------------------------------------------------------------- */

#include "arm_math.h"
#include "arm_common_tables.h"

 // * @ingroup groupFastMath    

 // * @addtogroup sin    

/**   
 * @brief Fast approximation to the trigonometric sine function for Q31 data.
 * @param[in] x Scaled input value in radians.
 * @return  sin(x).
 *
 * The Q31 input value is in the range [0 +0.9999] and is mapped to a radian value in the range [0 2*pi). */

q31_t arm_sin_q31(
  q31_t x)
{
  q31_t sinVal;                                  /* Temporary variables for input, output */
  int32_t index;                                 /* Index variables */
  q31_t a, b;                                    /* Four nearest output values */
  q31_t fract;                                   /* Temporary values for fractional values */

  // Calculate the nearest index 
  index = (uint32_t)x >> FAST_MATH_Q31_SHIFT;

  // Calculation of fractional value 
  fract = (x - (index << FAST_MATH_Q31_SHIFT)) << 9;

  // Read two nearest values of input value from the sin table 
  a = sinTable_q31[index];
  b = sinTable_q31[index+1];

  // Linear interpolation process 
  sinVal = (q63_t)(0x80000000-fract)*a >> 32;
  sinVal = (q31_t)((((q63_t)sinVal << 32) + ((q63_t)fract*b)) >> 32);

  return sinVal << 1;
}

 // *  end of sin group    
