/**
  ******************************************************************************
  * @file    OPAMP/OPAMP_PGA/main.c 
  * @author  MCD Application Team
  * @version V1.1.2
  * @date    14-August-2015
  * @brief   Main program body
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------
#include "stm32f30x.h"

 // @addtogroup STM32F30x_StdPeriph_Examples

 // @addtogroup OPAMP_PGA

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
#define DAC_DHR12R1_ADDRESS      0x40007414

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
const uint16_t Sine12bit[32] = {1023, 1223, 1415, 1592, 1749, 1875, 1969, 2028, 2047,
                                2028, 1969, 1875, 1747, 1592, 1415, 1223, 1023, 823,
                                631, 454, 299, 172, 77, 19, 0, 19, 77, 172, 299, 454,
                                631, 823};

// Private function prototypes -----------------------------------------------
static void OPAMP_Config(void);
static void DAC_Config(void);

// Private functions ---------------------------------------------------------


/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f30x.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f30x.c file
     */

  // DAC configuration 
  DAC_Config();

  // OPAMP1 configuration in PGA mode: gain = 2 
  OPAMP_Config();

  // forever loop 
  while(1);
}

/**
  * @brief  Configures OPAMP1 in PGA mode
  * @param  None
  * @retval None
  */
static void OPAMP_Config(void)
{
  OPAMP_InitTypeDef       OPAMP_InitStructure;
  GPIO_InitTypeDef        GPIO_InitStructure;

  /* PA5 is used as OPAMP1 non inverting input: it is already configured in
     analog mode in DAC_Config() */

  // GPIOA Peripheral clock is already enabled in DAC_Config() 

  // Configure PA2 analog mode: It is used as OPAMP1 output 
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  // OPAMP Peripheral clock enable 
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  // OPAMP1 config 
  OPAMP_InitStructure.OPAMP_NonInvertingInput = OPAMP_NonInvertingInput_IO2;
  OPAMP_InitStructure.OPAMP_InvertingInput =  OPAMP_InvertingInput_PGA;
  OPAMP_Init(OPAMP_Selection_OPAMP1, &OPAMP_InitStructure);

  // Configure OPAMP1 in PGA mode with gain set to 2 
  OPAMP_PGAConfig(OPAMP_Selection_OPAMP1, OPAMP_OPAMP_PGAGain_2, OPAMP_PGAConnect_No);

  // Enable OPAMP1 
  OPAMP_Cmd(OPAMP_Selection_OPAMP1, ENABLE);
}

/**
  * @brief  DAC configuration: configure DAC to be triggered by TIM2 using DMA
  *         transfer enabled
  * @param  None
  * @retval None
  */
static void DAC_Config(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  DAC_InitTypeDef DAC_InitStructure;
  DMA_InitTypeDef DMA_InitStructure;

  // TIM2 Periph clock enable 
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
  
  // Time base configuration 
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure); 
  TIM_TimeBaseStructure.TIM_Period = 0xFFFF;          
  TIM_TimeBaseStructure.TIM_Prescaler = 0x0;       
  TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;    
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  // TIM2 TRGO selection 
  TIM_SelectOutputTrigger(TIM2, TIM_TRGOSource_Update);

  // GPIOA clock enable 
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
  // Configure PA.05 (DAC_OUT2) as analog 
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_5;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  // DAC Periph clock enable 
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);
  // DAC channel2 Configuration 
  DAC_InitStructure.DAC_Trigger = DAC_Trigger_T2_TRGO;
  DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;
  DAC_InitStructure.DAC_LFSRUnmask_TriangleAmplitude = DAC_LFSRUnmask_Bit0;
  DAC_InitStructure.DAC_Buffer_Switch = DAC_BufferSwitch_Disable;
  DAC_Init(DAC1, DAC_Channel_2, &DAC_InitStructure);

  // Enable DAC Channel2 
  DAC_Cmd(DAC1, DAC_Channel_2, ENABLE);

  // DMA2 clock enable (to be used with DAC) 
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA2, ENABLE);

  // DMA channel4 Configuration 
  DMA_DeInit(DMA2_Channel4); 
  DMA_InitStructure.DMA_PeripheralBaseAddr = DAC_DHR12R1_ADDRESS;
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)&Sine12bit;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_BufferSize = 32;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA2_Channel4, &DMA_InitStructure);

  // Enable DMA2 Channel4 
  DMA_Cmd(DMA2_Channel4, ENABLE);

  // Enable DMA for DAC Channel2 
  DAC_DMACmd(DAC1, DAC_Channel_2, ENABLE);

  // TIM2 enable counter 
  TIM_Cmd(TIM2, ENABLE);
}


#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  // Infinite loop 
  while (1)
  {
  }
}
#endif



