/**
  ******************************************************************************
  * @file    RTC/RTC_LSI/main.c 
  * @author  MCD Application Team
  * @version V1.1.2
  * @date    14-August-2015
  * @brief   Main program body
  ******************************************************************************
  */


// Includes ------------------------------------------------------------------
#include "main.h"

 // @addtogroup STM32F30x_StdPeriph_Examples

 // @addtogroup RTC_LSI

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
RTC_InitTypeDef RTC_InitStructure;
__IO uint32_t LsiFreq = 0;
__IO uint32_t CaptureNumber = 0;

// Private function prototypes -----------------------------------------------
static void RTC_Config(void);
static uint32_t GetLSIFrequency(void);

// Private functions ---------------------------------------------------------

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f30x.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f30x.c file
     */

 // Initialize LEDs mounted on STM32303C-EVAL board --------------------------
  STM_EVAL_LEDInit(LED1);
  STM_EVAL_LEDInit(LED2);
  STM_EVAL_PBInit(BUTTON_KEY, BUTTON_MODE_GPIO);
  
  // RTC Configuration -------------------------------------------------------
  RTC_Config();

  // Wait Until KEY BUTTON is pressed 
  while(STM_EVAL_PBGetState(BUTTON_KEY) != RESET)
  {
  }
  
  // Get the LSI frequency:  TIM16 is used to measure the LSI frequency 
  LsiFreq = GetLSIFrequency();

  // Turn on LED2 
  STM_EVAL_LEDOn(LED2);

  // Calendar Configuration 
  RTC_InitStructure.RTC_AsynchPrediv = 88;
  RTC_InitStructure.RTC_SynchPrediv	=  (LsiFreq/89) - 1;
  RTC_InitStructure.RTC_HourFormat = RTC_HourFormat_24;
  RTC_Init(&RTC_InitStructure);

  // Infinite loop 
  while (1)
  {
  }
}

/**
  * @brief  Configure the RTC peripheral by selecting the clock source.
  * @param  None
  * @retval None
  */
static void RTC_Config(void)
{
  NVIC_InitTypeDef NVIC_InitStructure; 
  EXTI_InitTypeDef EXTI_InitStructure;
  
  // Enable the PWR clock 
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
  
  // Allow access to RTC 
  PWR_BackupAccessCmd(ENABLE);

// LSI used as RTC source clock 
// The RTC Clock may varies due to LSI frequency dispersion.    
  // Enable the LSI OSC  
  RCC_LSICmd(ENABLE);

  // Wait till LSI is ready   
  while(RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET)
  {
  }

  // Select the RTC Clock Source 
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSI);
   
  // Enable the RTC Clock 
  RCC_RTCCLKCmd(ENABLE);

  // Wait for RTC APB registers synchronisation 
  RTC_WaitForSynchro();

  // Calendar Configuration 
  RTC_InitStructure.RTC_AsynchPrediv = 88;
  RTC_InitStructure.RTC_SynchPrediv	=  470; /* (42KHz / 89) - 1 = 470 */
  RTC_InitStructure.RTC_HourFormat = RTC_HourFormat_24;
  RTC_Init(&RTC_InitStructure);  

  // EXTI configuration ******************************************************
  EXTI_ClearITPendingBit(EXTI_Line20);
  EXTI_InitStructure.EXTI_Line = EXTI_Line20;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);
  
  // Enable the RTC Wakeup Interrupt 
  NVIC_InitStructure.NVIC_IRQChannel = RTC_WKUP_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure); 
  
  // Configure the RTC WakeUp Clock source: CK_SPRE (1Hz) 
  RTC_WakeUpClockConfig(RTC_WakeUpClock_CK_SPRE_16bits);
  RTC_SetWakeUpCounter(0x0);

  // Enable the RTC Wakeup Interrupt 
  RTC_ITConfig(RTC_IT_WUT, ENABLE);
  
  // Enable Wakeup Counter 
  RTC_WakeUpCmd(ENABLE);
}

/**
  * @brief  Configures TIM16 to measure the LSI oscillator frequency. 
  * @param  None
  * @retval LSI Frequency
  */
static uint32_t GetLSIFrequency(void)
{
  NVIC_InitTypeDef   NVIC_InitStructure;
  TIM_ICInitTypeDef  TIM_ICInitStructure;
  RCC_ClocksTypeDef  RCC_ClockFreq;

  // TIM16 configuration ****************************************************** 
  // Enable TIM16 clock 
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM16, ENABLE);
  
  // Reset TIM16 registers 
  TIM_DeInit(TIM16);

  // Configure TIM16 prescaler 
  TIM_PrescalerConfig(TIM16, 0, TIM_PSCReloadMode_Immediate);

  // Connect internally the TIM16_CH1 to the RTC clock output 
  TIM16->OR = 0x3;
#if defined(STM32F303xC)
  RCC_MCOConfig(RCC_MCOSource_LSI);
#else /* STM32F334x8 || STM32F302x8 || STM32F303xE */
  RCC_MCOConfig(RCC_MCOSource_LSI, RCC_MCOPrescaler_1);
#endif /* STM32F303xC */

  /* TIM16 configuration: Input Capture mode ---------------------
     The reference clock(LSE or external) is connected to TIM16 CH1
     The Rising edge is used as active edge,
     The TIM16 CCR1 is used to compute the frequency value 
  ------------------------------------------------------------ */
  TIM_ICInitStructure.TIM_Channel     = TIM_Channel_1;
  TIM_ICInitStructure.TIM_ICPolarity  = TIM_ICPolarity_Rising;
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV8;
  TIM_ICInitStructure.TIM_ICFilter = 0x0;
  TIM_ICInit(TIM16, &TIM_ICInitStructure);

  // Enable the TIM16 global Interrupt 
  NVIC_InitStructure.NVIC_IRQChannel = TIM1_UP_TIM16_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  // Enable TIM16 counter 
  TIM_Cmd(TIM16, ENABLE);

  // Reset the flags 
  TIM16->SR = 0;
    
  // Enable the CC1 Interrupt Request   
  TIM_ITConfig(TIM16, TIM_IT_CC1, ENABLE);


  /* Wait until the TIM16 get 2 LSI edges (refer to TIM16_IRQHandler() in 
    stm32F30x_it.c file) ******************************************************/
  while(CaptureNumber != 2)
  {
  }
  // Deinitialize the TIM16 peripheral registers to their default reset values 
  TIM_DeInit(TIM16);


  // Compute the LSI frequency, depending on TIM16 input clock frequency (PCLK1)
  // Get SYSCLK, HCLK and PCLKx frequency 
  RCC_GetClocksFreq(&RCC_ClockFreq);
  
  // PCLK1 prescaler equal to 1 => TIMCLK = PCLK1 
  return (LsiFreq);
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  // Infinite loop 
  while (1)
  {
  }
}
#endif



