/**
  ******************************************************************************
  * @file    TIM/TIM_Synchronization/main.c  
  * @author  MCD Application Team
  * @version V1.1.2
  * @date    14-August-2015
  * @brief   Main program body
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------
#include "stm32f30x.h"

 // @addtogroup STM32F30x_StdPeriph_Examples

 // @addtogroup TIM_Synchronization

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
// Private function prototypes -----------------------------------------------
static void TIM_Config(void);
// Private functions ---------------------------------------------------------

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f30x.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f30x.c file
     */ 

  // TIMs Configuration 
  TIM_Config();

  // Infinite loop 
  while (1)
  {
  }
}

/**
  * @brief  Configure the TIMs.
  * @param  None
  * @retval None
  */
static void TIM_Config(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;
  
  // TIM2, TIM3 and TIM4 clock enable 
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2 | RCC_APB1Periph_TIM3 | RCC_APB1Periph_TIM4 , ENABLE);

  // GPIOA and GPIOB clocks enable 
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, ENABLE);

  // GPIOA Configuration: PA6(TIM3 CH1) as alternate function push-pull ------
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  // Connect TIM pins to AF2 
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_2);

  // GPIOA Configuration: PA0(TIM2 CH1) as alternate function push-pull ------
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  // Connect TIM pins to AF1 
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource0, GPIO_AF_1);

  // GPIOB Configuration: PB6(TIM4 CH1) as alternate function push-pull ------
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  // Connect TIM pins to AF2 
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_2);
  
  
  /* Timers synchronisation in cascade mode ----------------------------
     1/TIM2 is configured as Master Timer:
         - PWM Mode is used
         - The TIM2 Update event is used as Trigger Output  

     2/TIM3 is slave for TIM2 and Master for TIM4,
         - PWM Mode is used
         - The ITR1(TIM2) is used as input trigger 
         - Gated mode is used, so start and stop of slave counter
           are controlled by the Master trigger output signal(TIM2 update event).
         - The TIM3 Update event is used as Trigger Output. 

     3/TIM4 is slave for TIM3,
         - PWM Mode is used
         - The ITR2(TIM3) is used as input trigger
         - Gated mode is used, so start and stop of slave counter
           are controlled by the Master trigger output signal(TIM3 update event).

     In this example TIM2 input clock (TIM2CLK) is set to APB1 clock (PCLK1).   
     TIM2CLK = PCLK1  
     PCLK1 = HCLK 
     => TIM2CLK = HCLK = SystemCoreClock 

       The Master Timer TIM2 is running at TIM2 counter clock:
       TIM2 frequency = (TIM2 counter clock)/ (TIM2 period + 1) = 281.25 KHz 
       and the duty cycle = TIM2_CCR1/(TIM2_ARR + 1) = 25%.

       The TIM3 is running:
       - At (TIM2 frequency)/ (TIM3 period + 1) = 28.125 KHz and a duty cycle
         equal to TIM3_CCR1/(TIM3_ARR + 1) = 40%

	   The TIM4 is running:
       - At (TIM3 frequency)/ (TIM4 period + 1) = 7.031 KHz and a duty cycle
         equal to TIM4_CCR1/(TIM4_ARR + 1) = 50%
  
     Note: 
     SystemCoreClock variable holds HCLK frequency and is defined in system_stm32f30x.c file.
     Each time the core clock (HCLK) changes, user had to call SystemCoreClockUpdate()
     function to update SystemCoreClock variable value. Otherwise, any configuration
     based on this variable will be incorrect. 
  --------------------------------------------------------------------------- */

  // Time base configuration 
  TIM_TimeBaseStructure.TIM_Period = 255;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  TIM_TimeBaseStructure.TIM_Period = 9;
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

  TIM_TimeBaseStructure.TIM_Period = 3;
  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);

  // Init TIM_OCInitStructure 
  TIM_OCStructInit(&TIM_OCInitStructure);
  
  // Master Configuration in PWM1 Mode 
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 64;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC1Init(TIM2, &TIM_OCInitStructure);

  // Select the Master Slave Mode 
  TIM_SelectMasterSlaveMode(TIM2, TIM_MasterSlaveMode_Enable);

  // Master Mode selection 
  TIM_SelectOutputTrigger(TIM2, TIM_TRGOSource_Update);

  // Slaves Configuration: PWM1 Mode 
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 4;

  TIM_OC1Init(TIM3, &TIM_OCInitStructure);

  // Slave Mode selection: TIM3 
  TIM_SelectSlaveMode(TIM3, TIM_SlaveMode_Gated);
  TIM_SelectInputTrigger(TIM3, TIM_TS_ITR1);

  // Select the Master Slave Mode 
  TIM_SelectMasterSlaveMode(TIM3, TIM_MasterSlaveMode_Enable);

  // Master Mode selection 
  TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update);

  // Slaves Configuration: PWM1 Mode 
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 2;

  TIM_OC1Init(TIM4, &TIM_OCInitStructure);

  // Slave Mode selection: TIM4 
  TIM_SelectSlaveMode(TIM4, TIM_SlaveMode_Gated);
  TIM_SelectInputTrigger(TIM4, TIM_TS_ITR2);
 
  // TIM enable counter 
  TIM_Cmd(TIM3, ENABLE);
  TIM_Cmd(TIM2, ENABLE);
  TIM_Cmd(TIM4, ENABLE);
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  // Infinite loop 
  while (1)
  {
  }
}
#endif



